@@ -1,7 +1,8 @@
 #
 # This file is part of khmer, http://github.com/ged-lab/khmer/, and is
 # Copyright (C) Michigan State University, 2009-2013. It is licensed under
-# the three-clause BSD license; see doc/LICENSE.txt. Contact: ctb@msu.edu
+# the three-clause BSD license; see doc/LICENSE.txt.
+# Contact: khmer-project@idyll.org
 #
 import khmer
 
@@ -33,7 +34,7 @@ def test_readalign():
     graphAlign, readAlign = aligner.align(read)
 
     assert readAlign == 'ACCTAGGTTCGACATGTACC'
-    assert graphAlign == 'AGCTAGGTTCGACAAGT-CC'
+    assert graphAlign == 'AGCTAGGTTCGACAAGT-CC', graphAlign
     #                                   ^  ^
 
 
